/*
** Class:  LotROLinux.GLSDataCentre
** Author: Alan Jackson
** Date:   6th June 2007
**
** Class to extract the GLS server details using the supplied 
** url and game name for the game Lord of the Rings Online
**
** Based on a script by SNy <SNy@bmx-chemnitz.de>
**
** (C) 2007-2008 AJackson <ajackson@bcs.org.uk>
**
** This file is part of LotROLinux.
**
** LotROLinux is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** LotROLinux is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with LotROLinux.  If not, see <http://www.gnu.org/licenses/>.
*/
using System;
using System.IO;
using System.Net;
using System.Xml;
using System.Collections.Generic; 

namespace LotROLinux
{
	public class GLSDataCentre
	{
		private const string keyConfigFile = "GLSDataCenter.config";
		private const string keyDataCentre = "Datacenter";
		private const string keyWorlds = "Worlds";
		private const string keyWorldName = "Name";
		private const string keyWorldChatServer = "ChatServerUrl";
		private const string keyWorldServerStatus = "StatusServerUrl";
		private const string keyAuthServer = "AuthServer";
		private const string keyPatchServer = "PatchServer";
		private const string keyLaunchConfigServer = "LauncherConfigurationServer";
		private const string header = "SOAPAction: \"http://www.turbine.com/SE/GLS/GetDatacenters\"";
		private const string bodyDataTemplate = "<?xml version=\"1.0\" encoding=\"utf-8\"?><soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"><soap:Body><GetDatacenters xmlns=\"http://www.turbine.com/SE/GLS\"><game>{0}</game></GetDatacenters></soap:Body></soap:Envelope>";


		private string valAuthServer;
		private string valPatchServer;
		private string valLaunchConfigServer;
		private List<Realm> realmList;
		private bool flagLoadSuccess;

		public GLSDataCentre(string urlGLSDataCentreService, string gameName,
			string homeDir, string configDir)
		{
			realmList = null;

			Stream configStream;
			
			string bodyData = String.Format(bodyDataTemplate, gameName); 

			WebRequest dcRequest = WebRequest.Create (urlGLSDataCentreService);

			try
			{
				dcRequest.Method = "POST";
				dcRequest.ContentType = "text/xml; charset=utf-8";
			
				dcRequest.ContentLength = bodyData.Length;
				dcRequest.Headers.Add(header);

				Stream bodyStream = dcRequest.GetRequestStream();
				StreamWriter bodyStreamWrite = new StreamWriter(bodyStream);

				bodyStreamWrite.Write(bodyData);

				bodyStreamWrite.Flush();

				bodyStreamWrite.Close();
			
				bodyStream.Close();
				
				configStream = dcRequest.GetResponse().GetResponseStream();
				
				XmlDocument xDoc = new XmlDocument();
		
				xDoc.Load(configStream);

				xDoc.Save(String.Format("{0}{1}{2}", homeDir, configDir, keyConfigFile));

				XmlNodeList serverConfig = xDoc.GetElementsByTagName(keyDataCentre)[0].ChildNodes;

				foreach (XmlNode serverSetting in serverConfig)
				{
					if (serverSetting.Name.ToString() == keyWorlds)
					{
						XmlNodeList realms = serverSetting.ChildNodes;

						realmList = new List<Realm>();

						foreach (XmlNode realmSettings in realms)
						{
							string realmName = "";
							string realmChatServer = "";
							string realmServerStatus = "";
							
							foreach (XmlNode realmDetails in realmSettings.ChildNodes)
							{
								switch (realmDetails.Name.ToString())
								{
									case keyWorldName:
										realmName = realmDetails.InnerText;
										break;
									case keyWorldChatServer:
										realmChatServer = realmDetails.InnerText;
										break;
									case keyWorldServerStatus:
										realmServerStatus = realmDetails.InnerText;
										break;
								}
							}
							Realm tempRealm = new Realm(realmName,
								realmChatServer, realmServerStatus);

							realmList.Add(tempRealm);
						}
					} else {
						switch (serverSetting.Name.ToString())
						{
							case keyAuthServer:
								valAuthServer = serverSetting.InnerText;
								break;
							case keyPatchServer:
								valPatchServer = serverSetting.InnerText;
								break;
							case keyLaunchConfigServer:
								valLaunchConfigServer = serverSetting.InnerText;
								break;
						}
					}
				}
				
				flagLoadSuccess = true;
			}
			catch
			{
				dcRequest.Abort();
				flagLoadSuccess = false;
			}
		}

		public string GetAuthServer()
		{
			return valAuthServer;
		}
		
		public string GetPatchServer()
		{
			return valPatchServer;
		}
		
		public List<Realm> GetRealmList()
		{
			return realmList;
		}

		public string GetLaunchConfigServer()
		{
			return valLaunchConfigServer;
		}

		public bool IsLoadSuccess()
		{
			return flagLoadSuccess;
		}
	}
}
