/*
** Class:  LotROLinux.MainWindow
** Author: Alan Jackson
** Date:   6th June 2007
**
** Main windows for the linux based launcher
** for the game Lord of the Rings Online
**
** Based on a script by SNy <SNy@bmx-chemnitz.de>
**
** (C) 2007-2008 AJackson <ajackson@bcs.org.uk>
**
** This file is part of LotROLinux.
**
** LotROLinux is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** LotROLinux is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with LotROLinux.  If not, see <http://www.gnu.org/licenses/>.
*/
using System;
using Gtk;
using GLib;
using Glade;
using System.IO;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.Threading;
using System.Collections.Generic; 

namespace LotROLinux
{
	public class MainWindow
	{	
		[Widget]
        private Window winMain;

		[Widget]
		private MenuItem menuPatch;

		[Widget]
        private Image imgLOTRO;
        
		[Widget]
        private Alignment alignBrowser;
        
		[Widget]
        private Button btnLogin;
        
		[Widget]
        private ComboBox cboLanguage;
        
		[Widget]
        private ComboBox cboRealm;
        
		[Widget]
        private CheckButton chkSaveSettings;
        
		[Widget]
        private Entry txtAccount;
        
		[Widget]
        private Entry txtPasswd;
        
		[Widget]
        private TextView txtStatus;

		private const string aboutLotROLinux = "\n\t(C) 2007-2008 AJackson\n\nVersion ";
		private const string aboutCLI = "\n\nBased on CLI launcher for LOTRO\n\t(C) 2007-2008 by SNy";

		private string aboutVersion;
		private List<Game> gameList;
		private List<Realm> realmList;
		private List<Language> langList;
		private MozillaStatus webBrowser;
		private DetermineGame gameType;
		private Settings settings;
		private DetermineOS osType;
		private string valBaseDir;
		private string valHomeDir;
		private string urlAuthServer;
		private string urlWorldQueue;
		private string argWorldQueue;
		private string accNumber;
		private string accTicket;
		private string urlChatServer;
		private string urlLoginServer;
		private string clientName;
		private string clientArgs;
		private string urlNewsFeed;
		private string urlNewsStyleSheet;
		private string urlPatchServer;
		private string patchProductCode;
		private string gameName;
        private string icoFile;
        private string pngFile;
        private string configFile;
		private string title;
		private bool firstRun;
		private bool gameDirExists;

		public MainWindow(string version)
		{
			aboutVersion = version;
		}

		public void Run()
		{
			Glade.XML gxml = new Glade.XML (null, "MainWindow.glade", "winMain", null);
			gxml.Autoconnect (this);

			osType = new DetermineOS();

			valBaseDir = osType.AppDir;

			gameType = new DetermineGame();

			webBrowser = new MozillaStatus();
			alignBrowser.Add(webBrowser.GetBrowser());

			InitialSetup();
		}
		
		private void InitialSetup()
		{
			firstRun = true;

			gameDirExists = false;

			menuPatch.Visible = false;

			GetHomeDir();

			if (settings == null)
			{
				settings = new Settings(valHomeDir, osType);
			}
            
			txtAccount.Text = "";
			txtPasswd.Text = "";
			
			cboLanguage.Clear();
	        CellRendererText cellLanguage = new CellRendererText();
	        cboLanguage.PackStart(cellLanguage, false);
	        cboLanguage.AddAttribute(cellLanguage, "text", 0);
	        ListStore storeLanguage = new ListStore(typeof (string));
	        cboLanguage.Model = storeLanguage;
			cboLanguage.Active = 0;

			cboRealm.Clear();
	        CellRendererText cellRealm = new CellRendererText();
	        cboRealm.PackStart(cellRealm, false);
	        cboRealm.AddAttribute(cellRealm, "text", 0);
	        ListStore storeRealm = new ListStore(typeof (string));
	        cboRealm.Model = storeRealm;
			cboRealm.Active = 0;

			ClearLog();
			AddLog("Initialising, please wait...");

			if (!settings.LoadSettings())
			{
				AddLog("Error loading settings");
			} else {
				if (settings.FocusAccount())
				{
					txtAccount.HasFocus = true;
					chkSaveSettings.Active = false;
				} else {
					txtAccount.Text = settings.GetAccount();
					chkSaveSettings.Active = true;
					txtPasswd.HasFocus = true;
				}
			}

			gameType.GetSettings(settings.UsingDND(), settings.UsingTest());
			
            icoFile = gameType.GetICOFile();
			settings.SetIcoFile(icoFile);

            pngFile = gameType.GetPNGFile();

            configFile = String.Format("{0}{1}", settings.GetGameDir(), gameType.GetConfigFile());

			DirectoryInfo dir = new DirectoryInfo(settings.GetGameDir());

			if (dir.Exists)
			{
				gameDirExists = true;
			}

            title = gameType.GetTitle();

			winMain.Title = title.Replace("&amp;", "&");
			winMain.Icon = Gdk.Pixbuf.LoadFromResource(icoFile);
			imgLOTRO.Pixbuf = Gdk.Pixbuf.LoadFromResource(pngFile);

			GLib.Idle.Add (new IdleHandler (OnFirstRun));
		}

		private bool OnFirstRun()
	    {
			if (settings.AbortRun())
			{
				Application.Quit();
			} else {
				if (firstRun)
				{
					firstRun = false;

					if (!gameDirExists)
					{
						GameDirError();
					}

					LoadLanguageList();
				}
			}
			return firstRun;
		}
		
		protected void OnDeleteEvent (object sender, DeleteEventArgs a)
		{
			Application.Quit ();
			a.RetVal = true;
		}
		
		private void LoadLanguageList()
		{
			int setPos = 0;

			string setLanguage;
			string setGameDir;

			setLanguage = settings.GetLanguage();
			setGameDir = settings.GetGameDir();
			
			LanguageConfig langConfig = new LanguageConfig(setGameDir);
			
			if (langConfig.IsLanguageFound())
			{
				AddLog("Available languages checked.");
				
				langList = langConfig.GetLanguages();
				
				for (int i = 0; i < langList.Count; i++)
				{
					cboLanguage.AppendText(langList[i].GetName());
					if (langList[i].GetCode() == setLanguage)
					{
						setPos = i;
					}
				}
			
				cboLanguage.Active = setPos;

				LoadLauncherConfig();
			} else {
				AddLog("No language files found.");
			}
		}
		
		private void LoadLauncherConfig()
		{
			BaseConfig baseConfig = new BaseConfig(configFile);

			if (baseConfig.IsLauncherConfigOK())
			{
			    gameName = baseConfig.GetGameName();
			
				AccessGLSDataCentre(baseConfig.GetGLSDataCentreService(),
					gameName);
			} else {
	            configFile = String.Format("{0}{1}", settings.GetGameDir(), gameType.GetConfigFileAlt());

				baseConfig = new BaseConfig(configFile);

				if (baseConfig.IsLauncherConfigOK())
				{
					gameName = baseConfig.GetGameName();
			
					AccessGLSDataCentre(baseConfig.GetGLSDataCentreService(),
						gameName);
				} else {
					AddLog("Error reading launcher configuration file.");
				}
			}
		}

		private void AccessGLSDataCentre(string urlGLS, string gameName)
		{
			GLSDataCentre dataCentre = new GLSDataCentre(urlGLS, gameName, valHomeDir, valBaseDir);

			if (dataCentre.IsLoadSuccess())
			{
				urlAuthServer = dataCentre.GetAuthServer();
				urlPatchServer = dataCentre.GetPatchServer();

				AddLog("Fetched details from GLS data centre.");
				LoadRealms(dataCentre.GetRealmList());
				
				GetWorldQueueConfig(dataCentre.GetLaunchConfigServer());			
			} else {
				AddLog("Error accessing GLS data centre.");
			}
		}
		
		private void LoadRealms(List<Realm> realms)
		{
			int setPos = 0;
			string tmpName;
			string setRealm;

			setRealm = settings.GetRealm();

			realmList = realms;
			
			for (int i = 0; i < realmList.Count; i++)
			{
				tmpName = realmList[i].ServerName;
				cboRealm.AppendText(tmpName);

				if (tmpName == setRealm)
				{
					setPos = i;
				}
			}

			cboRealm.Active = setPos;
			AddLog("Realm list obtained.");
		}
		
		private void GetWorldQueueConfig(string urlWorldQueueServer)
		{
			WorldQueueConfig worldQueueConfig = new WorldQueueConfig(urlWorldQueueServer,
				valHomeDir, valBaseDir);

			if (worldQueueConfig.IsLoadSuccess())
			{
				urlWorldQueue = worldQueueConfig.GetWorldQueueURL();
				argWorldQueue = worldQueueConfig.GetWorldQueueParam();
				clientName = worldQueueConfig.GetGameFilename();
				clientArgs = worldQueueConfig.GetGameArgTemplate();
				urlNewsFeed = worldQueueConfig.GetNewsFeedURL();
				urlNewsStyleSheet = worldQueueConfig.GetNewsStyleSheetURL();
				patchProductCode = worldQueueConfig.GetPatchProductCode();
		
				System.Threading.Thread newsThread = new System.Threading.Thread(GetNews);
				newsThread.Start();

				AddLog("World queue configuration read");
				
				menuPatch.Visible = true;
				btnLogin.Visible = true;
				chkSaveSettings.Visible = true;
			} else {
				AddLog("Error getting world queue configuration");
			}
		}
		
		private void GetNews()
		{
			string htmlFileName = String.Format("{0}{1}/LotROLinux.html", valHomeDir, valBaseDir);

			if (urlNewsFeed.Contains("{lang}"))
			{
				urlNewsFeed = urlNewsFeed.Replace("{lang}", langList[cboLanguage.Active].GetCode().Replace("_", "-"));
			}

			try
			{
				XmlUrlResolver resolver = new XmlUrlResolver();

				XslTransform xslt = new XslTransform();

				xslt.Load(urlNewsStyleSheet, resolver);

				xslt.Transform(urlNewsFeed, htmlFileName, resolver);

				Gtk.Application.Invoke (delegate {
					ShowNews(htmlFileName);
				});
			}
			catch
			{
				AddLog("Error fetching news stream");
			}
		}
		
		private void ShowNews(string newsFileName)
		{
			const string htmlType = "file://";
			
			webBrowser.ShowUrl(String.Format("{0}{1}", htmlType, newsFileName));
		}
		
		private void GetHomeDir()
		{
			valHomeDir = System.Environment.GetEnvironmentVariable("HOME");

			if (!valHomeDir.EndsWith("/"))
			{
				valHomeDir += "/";
			}
		}

		private void AppendTextWithoutScroll (string text) 
		{
			Gtk.TextBuffer buf = txtStatus.Buffer;

			Gtk.TextIter iter;
			buf.MoveMark(buf.InsertMark, buf.EndIter);
			if(text != null) {
				if (text.Equals ("") == false) {				
					iter = buf.EndIter;
					buf.Insert (ref iter, text);
				}
			}
			iter = buf.EndIter;
			buf.Insert (ref iter, "\n");
			buf.MoveMark (buf.SelectionBound, buf.EndIter);
		}
		
		private void ClearLog()
		{
			txtStatus.Buffer.Text = "";
		}
		
		private void AddLog(string message)
		{
			Gtk.TextBuffer buf = txtStatus.Buffer;

			AppendTextWithoutScroll(message);

			while (Gtk.Application.EventsPending ()) 
				Gtk.Application.RunIteration ();

			txtStatus.ScrollToMark (buf.InsertMark, 0.4, true, 0.0, 1.0);
		}
		
		private void AuthAccount()
		{
			AuthenticateUser account = new AuthenticateUser(urlAuthServer,
				txtAccount.Text, txtPasswd.Text, gameName, valHomeDir, valBaseDir);

			if (account.isAuthenticated())
			{
				AddLog("Account authenticated");

				gameList = account.GetGameList();

				if (gameList.Count > 1)
				{
					GameSelectWindow newWizard = new GameSelectWindow(gameList);

					newWizard.Run();

					accNumber = newWizard.GetGameName();

					newWizard.Destroy();
				} else {
					accNumber = gameList[0].GetName();
				}

				accTicket = account.GetTicket();
				GetRealmDetails();
			} else {
				AddLog(account.GetErrorMessage());
			}
		}
		
		private void GetRealmDetails()
		{
			Realm realm = realmList[cboRealm.Active];
			
			realm.CheckRealm(settings.UsingDND(), valHomeDir, valBaseDir);
			
			if (realm.IsRealmAvailable)
			{
				urlChatServer = realm.ChatServerURL;
				urlLoginServer = realm.LoginServer;
				
				if (settings.UsingDND())
				{
					StartClient();
				} else {
					if (realm.QueueURL == "")
					{
						StartClient();
					} else {
						EnterWorldQueue(realm.QueueURL);
					}
				}
			} else {
				AddLog("Error getting realm status");
			}
		}

		private void EnterWorldQueue(string urlRealmQueue)
		{
			JoinWorldQueue worldQueue = new JoinWorldQueue(argWorldQueue,
				accNumber, accTicket, urlRealmQueue, urlWorldQueue,
				valHomeDir, valBaseDir);

			if (worldQueue.IsJoinSuccess)
			{
				AddLog("Joined world queue");
				
				InQueue(worldQueue.QueueNumber, worldQueue.NowServing);
			} else {
				AddLog("Error joining world queue");
			}
		}

		private void InQueue(string queueNumber, string nowServing)
		{
			string serving = nowServing;

			if (String.Compare(queueNumber, serving) > 0)
			{
				AddLog("Currently queueing, please wait...");
			}

			while (String.Compare(queueNumber, serving) > 0)
			{
				Realm realm = realmList[cboRealm.Active];
			
				realm.CheckRealm(settings.UsingDND(), valHomeDir, valBaseDir);
			
				if (realm.IsRealmAvailable)
				{
					serving = realm.NowServing;
				} else {
					AddLog("Error getting realm status");
					break;
				}
			}

			if (String.Compare(queueNumber, serving) <= 0)
			{
				StartClient();
			}
		}

		private void StartClient()
		{
			winMain.Hide();

			AddLog("Starting game");

			StartGame game = new StartGame(clientName, clientArgs, accNumber,
				urlLoginServer, accTicket, urlChatServer,
				langList[cboLanguage.Active].GetCode(),
				settings.GetGameDir(), settings.GetWineProg(),
				settings.GetWineParams(), settings.GetWineDebug(),
				settings.GetWinePrefix(), valHomeDir, 
				settings.GetWineLog(), settings.HiResEnabled(),
				settings.GetApp(), osType);

			game.Run();

			Application.Quit();
		}

		private void GameDirError()
		{
			MessageDialog md = new MessageDialog (winMain,
				DialogFlags.DestroyWithParent,
				MessageType.Error,  ButtonsType.Ok, 
				String.Format("Can not find the directory\n{0}\nPlease check game directory under options.",
					settings.GetGameDir()));
	   		md.Title = "Game Directory Error";
			md.Run ();
			md.Destroy();	
		}

		protected virtual void OnBtnLoginClicked(object sender, System.EventArgs e)
		{
			if ((txtAccount.Text == "") || (txtPasswd.Text == ""))
			{
				AddLog("Please enter account name and password");
			} else {
				settings.SetRealm(cboRealm.ActiveText);
				settings.SetLanguage(langList[cboLanguage.Active].GetCode());
				settings.SetAccount(txtAccount.Text);

				if (settings.SaveSettings(chkSaveSettings.Active))
				{
					AuthAccount();
				} else {
					AddLog("Error saving settings");
				}
			}
		}

		protected virtual void OnAboutActivated(object sender, System.EventArgs e)
		{
			MessageDialog md = new MessageDialog (winMain,
			DialogFlags.DestroyWithParent,
			MessageType.Info,  ButtonsType.Ok, 
			title + aboutLotROLinux + aboutVersion + aboutCLI);
	   		md.Title = "About";
			md.Run ();
			md.Destroy();	
		}

		protected virtual void OnExitActivated(object sender, System.EventArgs e)
		{
			Application.Quit();
		}

		protected virtual void OnTxtAccountActivated(object sender, System.EventArgs e)
		{
			txtPasswd.HasFocus = true;
		}

		protected virtual void OnTxtPasswdActivated(object sender, System.EventArgs e)
		{
			if (btnLogin.Visible)
			{
				btnLogin.Click();
			}
		}

		protected virtual void OnOptionsActivated(object sender, System.EventArgs e)
		{
			settings.ModifySettings();

			if (settings.SaveSettings(chkSaveSettings.Active))
			{
				if (settings.GameDirChanged())
				{
					InitialSetup();
				}
			} else {
				AddLog("Error saving settings");
			}
		}

		protected virtual void OnPatchActivated(object sender, System.EventArgs e)
		{
			PatchWindow winPatch = new PatchWindow(urlPatchServer, patchProductCode,
				langList[cboLanguage.Active].GetPatchCode(),
				settings.GetGameDir(), settings.GetPatchClient(), settings.GetWineProg(),
				settings.HiResEnabled(), icoFile, valHomeDir, settings.GetWinePrefix(), 
				settings.GetApp(), osType);
			
			winPatch.Show();
		}

		protected virtual void OnSwitchGameActivated (object sender, System.EventArgs e)
		{
			if (settings.SwitchGame())
			{
				InitialSetup();
			}
		}

		protected virtual void OnCboRealmChanged (object sender, System.EventArgs e)
		{
			if (cboRealm.Active >= 0)
			{
				settings.SetRealm(cboRealm.ActiveText);
			}
		}

		protected virtual void OnCboLanguageChanged (object sender, System.EventArgs e)
		{
			if (cboLanguage.Active >= 0)
			{
				settings.SetLanguage(langList[cboLanguage.Active].GetCode());
			}
		}
	}
}
